import 'dart:convert';
import 'dart:ui';
import 'package:meta/meta.dart';
import '../../models/rollee_connect_config.dart';
import '../../models/rollee_connect_initial_state.dart';
import '../../models/rollee_connect_params.dart';
import '../constants/constants.dart';
import "package:http/http.dart" as http;

import 'logger.dart';

/// This class represents helpers for the SDK.
@immutable
abstract class RolleeHelpers {
  /// This method returns the URL based on the [type] of URL and the [isProduction] flag.
  ///
  /// example:
  /// ```dart
  /// final url = RolleeHelpers.makeURL(
  ///  type: RolleeUrlType.config,
  ///  isProduction: false,
  /// );
  /// print(url); // ...
  /// ```
  static String makeURL({
    required RolleeUrlType type,
    required bool isProduction,
  }) {
    String url = isProduction
        ? productionUrlBasedOnUrlType(type)
        : developmentUrlBasedOnUrlType(type);

    url += _endpointBasedOnType(type);

    RolleeLogger.connectLog("URL created: $url");

    return url;
  }

  /// Generates an initial state for the [RolleeConnect] widget.
  /// This method is used internally by the SDK.
  static RolleeConnectInitialState generateInitialConfig({
    required RolleeConnectConfig config,
  }) {
    final state = RolleeConnectInitialState(
      width: 0,
      height: 0,
      token: config.token,
      sessionToken: config.sessionToken,
      isProduction: config.setup?.isProduction ?? false,
      params: RolleeConnectParams(
        introScreenTitle: config.setup?.introScreenTitle,
        ctaButtonForeColor: hexFromFlutterColor(config.setup?.ctaTextColor),
        ctaButtonBackColor:
            hexFromFlutterColor(config.setup?.ctaBackgroundColor),
        partnerLogo: config.setup?.partnerLogoUrl,
        redirect: config.setup?.redirectUrl,
      ),
    );

    RolleeLogger.connectLog("Initial configuration created.");

    return state;
  }

  /// Generates a product URL for the [RolleeConnect] view widget, based on the [config] and the [isProduction] flag.
  ///
  /// example:
  /// ```dart
  /// final url = RolleeHelpers.generateProductTypeUrl(
  /// config: RolleeConnectConfig(...),
  /// isProduction: false,
  /// );
  /// print(url); // ...
  /// ```
  static String generateProductTypeUrl({
    required RolleeConnectConfig config,
    required bool isProduction,
  }) {
    const type = RolleeUrlType.session;

    final introScreenSkip =
        'skip_first_page=${config.setup?.introScreenSkip ?? false}';

    final skipDataSourcesPage =
        'skip_datasources_page=${config.setup?.skipDataSourcesPage ?? false}';

    final removeCloseButton =
        'remove_close_button=${config.setup?.removeCloseButton ?? false}';

    final datasource = config.setup?.dataSource != null
        ? "datasource=${config.setup!.dataSource}"
        : null;

    String madeUrl = makeURL(
      type: type,
      isProduction: isProduction,
    );
    if (type == RolleeUrlType.session) {
      madeUrl += config.sessionToken;
    }

    madeUrl +=
        "?$introScreenSkip&$skipDataSourcesPage&$removeCloseButton&$datasource";

    RolleeLogger.connectLog("Product URL created: $madeUrl");

    return madeUrl;
  }

  /// This methods sets a configuration state remotely to your account with the [state] provided.
  static Future<void> setConfiguration(
      RolleeConnectConfig configuration) async {
    final initialConfig = generateInitialConfig(config: configuration);

    final params = {
      ...initialConfig.params.toMap(),
    };
    final keys = params.keys.toList();

    final config = keys.fold<Map<String, dynamic>>({}, (acc, key) {
      if (params[key] != null) {
        acc[key] = params[key]!;
      }
      return acc;
    });

    try {
      final madeUrl = makeURL(
        type: RolleeUrlType.config,
        isProduction: initialConfig.isProduction,
      );
      final response = await http.post(
        Uri.parse(madeUrl),
        headers: {
          'Authorization': 'Bearer ${initialConfig.token}',
          'Content-Type': 'application/json',
        },
        body: jsonEncode({"config": config}),
      );

      if (response.statusCode >= 200 && response.statusCode < 300) {
        RolleeLogger.connectLog("Configuration updated successfully.");
      } else {
        RolleeLogger.connectLog("Configuration update failed.");
      }
    } catch (e) {
      RolleeLogger.connectLog("Configuration update failed.");
    }
  }

  /// This method returns the URL based on the [type] of URL and the [isProduction] flag.
  static String developmentUrlBasedOnUrlType(RolleeUrlType type) {
    if (type == RolleeUrlType.config) {
      RolleeLogger.connectLog("Current mode: sandbox");

      return RolleeConstants.api.sandbox;
    } else {
      assert(
        type == RolleeUrlType.session,
        "Invalid or unsupported URL type.",
      );
      return RolleeConstants.base.sandbox;
    }
  }

  /// This method returns a production URL based on the [type] of URL.
  static String productionUrlBasedOnUrlType(RolleeUrlType type) {
    if (type == RolleeUrlType.config) {
      RolleeLogger.connectLog("Current mode: production");

      return RolleeConstants.api.production;
    } else {
      assert(
        type == RolleeUrlType.session,
        "Invalid or unsupported URL type.",
      );
      return RolleeConstants.base.production;
    }
  }

  /// This method returns the endpoint based on the [type] of URL.
  static String _endpointBasedOnType(RolleeUrlType type) {
    switch (type) {
      case RolleeUrlType.config:
        return RolleeConstants.config;
      case RolleeUrlType.session:
        return RolleeConstants.session;
      default:
        throw Exception("Invalid or unsupported URL type.");
    }
  }

  /// This method returns the hex value of a [Color].
  static String? hexFromFlutterColor(Color? ctaTextColor) {
    if (ctaTextColor == null) {
      return null;
    }

    return '#${ctaTextColor.value.toRadixString(16).substring(2)}';
  }

  /// Whether the provided [text] is a valid URL or not.
  static bool isValidUrl(String text) {
    try {
      Uri.parse(text);
      return true;
    } on FormatException {
      RolleeLogger.connectLog("Invalid URL: $text");
      return false;
    } catch (e) {
      RolleeLogger.connectLog("Invalid URL: $text");
      return false;
    }
  }

  const RolleeHelpers._();
}
