import 'package:flutter/material.dart';
import 'package:rollee_flutter_sdk/core/utils/constants/constants.dart';
import 'package:rollee_flutter_sdk/core/utils/helpers/channel.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../core/models/rollee_connect_config.dart';
import '../core/models/rollee_connect_login_result.dart';
import '../core/models/rollee_connect_result.dart';
import '../core/utils/helpers/helpers.dart';

/// {@macro rollee_connect}
class RolleeConnect extends StatefulWidget {
  /// {@template rollee_connect}
  /// This widget represents the Rollee Connect view, you can use it to show the Rollee Connect view.
  /// the [config] accepts a [RolleeConnectConfig] object, which contains the configuration for the Rollee Connect view.
  ///
  ///
  /// the [onCompleted] function is called when the Rollee Connect flow is completed.
  ///
  ///
  /// the [onClose] function is called when the Rollee Connect view is closed.
  ///
  ///
  /// the [onLoginSuccess] function is called when the user is logged successfully.
  ///
  /// the [onHomeStarted] function is called when the home page starts.
  ///
  /// the [onTermsStarted] function is called when the terms page starts.
  ///
  /// the [onSearchStarted] function is called when the search page starts.
  ///
  /// the [onLoginStarted] function is called when the login page starts.
  ///
  /// the [onFetchStarted] function is called when the fetching of the user data starts.
  ///
  /// the [onConnected] function is called when the connected page starts.
  ///

  ///rollee_flutter_sdkrollee_flutter_sdk
  /// example:
  /// ```dart
  /// RolleeConnect(
  ///   config: const RolleeConnectConfig(
  ///     token: "<YOUR-TOKEN-HERE>",
  ///     sessionToken: "<THE-SESSION-TOKEN-HERE>",
  ///   ),
  ///   onCompleted: (result) {
  ///     print("onCompleted $result");
  ///   },
  ///   onClose: (close) {
  ///     print("onClose $close");
  ///   },
  ///   onLoginSuccess: (loginResult) {
  ///     print("onLoginSuccess $loginResult");
  ///   },
  /// ),
  /// ```
  /// {@endtemplate}
  const RolleeConnect({
    super.key,
    required this.config,
    this.onCompleted,
    this.onClose,
    this.onLoginSuccess,
    this.onHomeStarted,
    this.onTermsStarted,
    this.onSearchStarted,
    this.onLoginStarted,
    this.onFetchStarted,
    this.onConnected,
  });

  /// {@macro rollee_connect_config}
  final RolleeConnectConfig config;

  /// This function is called when the Rollee Connect view is completed.
  final void Function(RolleeConnectResult result)? onCompleted;

  /// This function is called when the user is logged successfully.
  final void Function(RolleeConnectLoginResult loginResult)? onLoginSuccess;

  /// This function is called when the Rollee Connect view is closed.
  final void Function(bool close)? onClose;

  /// This function is called when the home page starts.
  final void Function()? onHomeStarted;

  /// This function is called when the terms page starts.
  final void Function()? onTermsStarted;

  /// This function is called when the search page starts.
  final void Function()? onSearchStarted;

  /// This function is called when the login page starts.
  final void Function()? onLoginStarted;

  /// This function is called when the fetching of the user data starts.
  final void Function()? onFetchStarted;

  /// This function is called when the connected page starts.
  final void Function()? onConnected;

  @override
  State<RolleeConnect> createState() => RolleeConnectState();
}

class RolleeConnectState extends State<RolleeConnect> {
  /// The responsible controller for the webview.
  WebViewController? controller;

  /// This method is called when the widget is initialized & updated.
  Future<void> widgetSideEffect() async {
    if (widget.config.token.isNotEmpty) {
      await RolleeHelpers.setConfiguration(widget.config);
    }
  }

  @override
  void initState() {
    super.initState();
    // the controller should be initialized only once, here.
    controller = WebViewController();

    // before we can  load the URL, we need to ensure that the configuration is set, so we call the widgetSideEffect method and wait for it to finish, then we can load the URL in the webview.
    widgetSideEffect().then((e) {
      String urlToOpen = RolleeHelpers.generateProductTypeUrl(
        config: widget.config,
        isProduction: widget.config.setup?.isProduction ?? false,
      );

      // By default the webview is in disabled mode, which means that the webview will not allow any javascript to be executed, so we need to set the mode to unrestricted.
      controller!.setJavaScriptMode(JavaScriptMode.unrestricted);

      // we load the URL in the webview.
      controller!.loadRequest(Uri.parse(urlToOpen));

      // we disable the zoom in the webview, we need to disable the zoom to prevent the user from zooming in the webview.
      controller!.enableZoom(false);

      // we add the javascript channel to the webview, this is the channel that will be used to communicate between the webview and the flutter app, to trigger callbacks.
      controller!.addJavaScriptChannel(
        RolleeConstants.channelName,
        onMessageReceived: (JavaScriptMessage jsMessage) =>
            RolleeChannel.onMessage(
          jsMessage,
          config: widget.config,
          onCompletedHandler: widget.onCompleted ?? (result) {},
          onCloseHandler: widget.onClose ?? (close) {},
          onLoginSuccessHandler: widget.onLoginSuccess ?? (loginResult) {},
          onConnectedHandler: widget.onConnected ?? () {},
          onFetchHandler: widget.onFetchStarted ?? () {},
          onHomeHandler: widget.onHomeStarted ?? () {},
          onLoginHandler: widget.onLoginStarted ?? () {},
          onSearchHandler: widget.onSearchStarted ?? () {},
          onTermsHandler: widget.onTermsStarted ?? () {},
        ),
      );
    });
  }

  @override
  void didUpdateWidget(covariant RolleeConnect oldWidget) {
    if (widget.config.token.isNotEmpty && oldWidget.config != widget.config) {
      // if the configuration of the widget is updated, we need to update the configuration in the webview as well (set the new configuration).
      widgetSideEffect();
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    // when the widget is disposed, we remove the javascript channel from the webview.
    controller!.removeJavaScriptChannel(RolleeConstants.channelName);
    // controller!.clearCache();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    //! Don't add any logic here, this is a wrapper for the WebViewWidget.
    return WebViewWidget(
      controller: controller!,
    );
  }
}
