import 'dart:ui';
import 'package:rollee_flutter_sdk/core/utils/helpers/channel.dart';
import 'package:test/test.dart';

import 'package:rollee_flutter_sdk/core/models/export.dart';
import 'package:rollee_flutter_sdk/core/utils/constants/constants.dart';
import 'package:rollee_flutter_sdk/core/utils/helpers/helpers.dart';
import 'package:webview_flutter/webview_flutter.dart';

void main() {
  group('RolleeHelpers Tests', () {
    test('Test makeURL()', () {
      expect(
        RolleeHelpers.makeURL(
          type: RolleeUrlType.config,
          isProduction: false,
        ),
        'https://api.sand.getrollee.com/api/v0.3/connect/config',
      );
      expect(
        RolleeHelpers.makeURL(
          type: RolleeUrlType.config,
          isProduction: true,
        ),
        'https://api.getrollee.com/api/v0.3/connect/config',
      );
      expect(
        RolleeHelpers.makeURL(
          type: RolleeUrlType.session,
          isProduction: false,
        ),
        'https://connectv2.sand.getrollee.com/v0.4/session/',
      );
      expect(
        RolleeHelpers.makeURL(
          type: RolleeUrlType.session,
          isProduction: true,
        ),
        'https://connectv2.getrollee.com/v0.4/session/',
      );
    });

    test("Test generateInitialConfig()", () {
      final initialConfig = RolleeHelpers.generateInitialConfig(
        config: RolleeConnectConfig(
          sessionToken: "sessionToken",
          token: "token",
          setup: RolleeConnectConfigSetup(
            isProduction: false,
          ),
        ),
      );
      expect(initialConfig, isA<RolleeConnectInitialState>());
      expect(initialConfig.sessionToken, "sessionToken");
      expect(initialConfig.token, "token");
      expect(initialConfig.isProduction, false);
    });

    test(
      "Test generateProductTypeUrl()",
      () {
        String url = RolleeHelpers.generateProductTypeUrl(
          isProduction: false,
          config: const RolleeConnectConfig(
            sessionToken: 'sessionToken',
            token: 'token',
          ),
        );

        expect(url, contains("skip_first_page=false"));
        expect(url, contains("skip_datasources_page=false"));
        expect(url, contains("remove_close_button=false"));
        expect(url, isNot(contains("datasource=")));
        expect(url, contains("sessionToken"));

        url = RolleeHelpers.generateProductTypeUrl(
          isProduction: true,
          config: RolleeConnectConfig(
            sessionToken: 'sessionToken',
            token: 'token',
            setup: RolleeConnectConfigSetup(
              skipDataSourcesPage: true,
              isProduction: true,
              ctaBackgroundColor: const Color(0xFF000000),
              ctaTextColor: const Color(0xFFcdc8fd),
              introScreenSkip: true,
              introScreenTitle: "introScreenTitle",
              partnerLogoUrl: "partnerLogoUrl",
              redirectUrl: "redirectUrl",
              removeCloseButton: true,
              dataSource: "dataSourceId",
            ),
          ),
        );
        expect(url, contains("skip_first_page=true"));
        expect(url, contains("skip_datasources_page=true"));
        expect(url, contains("remove_close_button=true"));
        expect(url, contains("datasource=dataSourceId"));
        expect(url, contains("sessionToken"));
      },
    );
    test("Test hexFromFlutterColor()", () {
      final hex = RolleeHelpers.hexFromFlutterColor(
        const Color(0xFF000000),
      );
      expect(hex, "#000000");

      final hex2 = RolleeHelpers.hexFromFlutterColor(
        const Color(0xFFcdc8fd),
      );
      expect(hex2, "#cdc8fd");

      //! assert(Color.purple == Color(0xFF9C27B0)); This is true, the only reason it is commented out is because it require the import of flutter/material.dart, which we don't want to do in the test file.
      final hex3 = RolleeHelpers.hexFromFlutterColor(
        const Color(0xFF9C27B0),
      );
      expect(hex3?.toUpperCase(), "#9C27B0");
    });
    test('Test setConfiguration()', () async {
      try {
        await RolleeHelpers.setConfiguration(
          const RolleeConnectConfig(
            sessionToken: "sessionToken",
            token: "token",
          ),
        );
      } catch (e) {
        // should not throw an error here.
        assert(false);
      }
    });
  });
  group(
    "Models Equality",
    () {
      test('Test models Equality', () {
        const connectResultA = RolleeConnectResult(
          isCompleted: true,
          userUUID: "UUID",
        );
        const connectResultB = RolleeConnectResult(
          isCompleted: true,
          userUUID: "UUID",
        );
        expect(connectResultA == connectResultB, true);

        const paramsA = RolleeConnectParams();
        const paramsB = RolleeConnectParams();
        expect(paramsA == paramsB, true);

        const loginResultA = RolleeConnectLoginResult(
          accountId: "accountId",
          dataSourceId: "dataSourceId",
          isCompleted: true,
        );
        const loginResultB = RolleeConnectLoginResult(
          accountId: "accountId",
          dataSourceId: "dataSourceId",
          isCompleted: true,
        );
        expect(loginResultA == loginResultB, true);

        const initialStateA = RolleeConnectInitialState(
          isProduction: true,
          height: 0,
          width: 0,
          params: paramsA,
          sessionToken: "sessionToken",
          token: "token",
        );

        const initialStateB = RolleeConnectInitialState(
          isProduction: true,
          height: 0,
          width: 0,
          params: paramsB,
          sessionToken: "sessionToken",
          token: "token",
        );

        expect(initialStateA == initialStateB, true);

        final configSetupA = RolleeConnectConfigSetup(
          isProduction: true,
          skipDataSourcesPage: true,
          ctaBackgroundColor: const Color(0xFF000000),
          ctaTextColor: const Color(0xFFcdc8fd),
          introScreenSkip: true,
          introScreenTitle: "introScreenTitle",
          partnerLogoUrl: "partnerLogoUrl",
          redirectUrl: "redirectUrl",
          removeCloseButton: true,
          dataSource: "dataSourceId",
        );
        final configSetupB = RolleeConnectConfigSetup(
          isProduction: true,
          skipDataSourcesPage: true,
          ctaBackgroundColor: const Color(0xFF000000),
          ctaTextColor: const Color(0xFFcdc8fd),
          introScreenSkip: true,
          introScreenTitle: "introScreenTitle",
          partnerLogoUrl: "partnerLogoUrl",
          redirectUrl: "redirectUrl",
          removeCloseButton: true,
          dataSource: "dataSourceId",
        );

        expect(configSetupA == configSetupB, true);

        final configA = RolleeConnectConfig(
          sessionToken: "sessionToken",
          token: "token",
          setup: configSetupA,
        );
        final configB = RolleeConnectConfig(
          sessionToken: "sessionToken",
          token: "token",
          setup: configSetupB,
        );

        expect(configA == configB, true);
      });
    },
  );

  group(
    'Test channel\'s onMessage',
    () {
      test(
        'Test onMessage ',
        () {
          Stream<JavaScriptMessage> mockStream = Stream.fromIterable(
            [
              const JavaScriptMessage(message: "closeSession"),
              const JavaScriptMessage(
                message:
                    "{\"onLoginSuccess\":true,\"datasourceID\":\"a3b1f690-ea1b-4d3c-948b-f315d03487b0\",\"accountID\":\"c93325b1-4a80-4c3f-92d9-9f08296208cc\"}",
              ),
              const JavaScriptMessage(
                message:
                    "{\"onCompleted\": true, \"userUUID\": \"d711d210-2ebe-4cce-97aa-e3d6992dff9f\"}",
              ),
              const JavaScriptMessage(message: "Home"),
              const JavaScriptMessage(message: "Terms"),
              const JavaScriptMessage(message: "Search"),
              const JavaScriptMessage(message: "Login"),
              const JavaScriptMessage(message: "Fetch"),
              const JavaScriptMessage(message: "Connected"),
            ],
          );

          mockStream.listen(
            (event) {
              RolleeChannel.onMessage(
                event,
                config: const RolleeConnectConfig(
                  sessionToken: "sessionToken",
                  token: "token",
                ),
                onCompletedHandler: (result) {
                  expect(
                    event.message ==
                        "{\"onCompleted\": true, \"userUUID\": \"d711d210-2ebe-4cce-97aa-e3d6992dff9f\"}",
                    true,
                  );
                  expect(result.isCompleted, true);
                  expect(
                      result.userUUID, "d711d210-2ebe-4cce-97aa-e3d6992dff9f");
                },
                onLoginSuccessHandler: (loginResult) {
                  expect(
                    event.message ==
                        "{\"onLoginSuccess\":true,\"datasourceID\":\"a3b1f690-ea1b-4d3c-948b-f315d03487b0\",\"accountID\":\"c93325b1-4a80-4c3f-92d9-9f08296208cc\"}",
                    true,
                  );
                  expect(loginResult.accountId,
                      "c93325b1-4a80-4c3f-92d9-9f08296208cc");
                  expect(loginResult.dataSourceId,
                      "a3b1f690-ea1b-4d3c-948b-f315d03487b0");
                  expect(loginResult.isCompleted, true);
                },
                onCloseHandler: (close) {
                  expect(event.message == "closeSession", true);
                  expect(close, true);
                },
                onConnectedHandler: () {
                  expect(event.message == "Connected", true);
                },
                onFetchHandler: () {
                  expect(event.message == "Fetch", true);
                },
                onHomeHandler: () {
                  expect(event.message == "Home", true);
                },
                onLoginHandler: () {
                  expect(event.message == "Login", true);
                },
                onSearchHandler: () {
                  expect(event.message == "Search", true);
                },
                onTermsHandler: () {
                  expect(event.message == "Terms", true);
                },
              );
            },
          );
        },
      );
    },
  );
}
